/***********************************************************************

nucular -- a program that splits extended pedigrees into nuclear families
Copyright (C) 2005 Mathieu Lemire.

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**********************************************************************/


//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890


#include "PedStruct.hh"	

PedStruct::PedStruct( char * filename,  
		      int n_lines_to_skip ){


  char * ligne = new char [ L_LENGTH ];
  char * foochar = new char [ W_LENGTH ];
  long foolong;
  int fooint, i, n_lignes=0;
  bool header_line=1;
  string tamponoutadr;

    
  n_header_lines = 0;

  if( DEBUG_PEDSTRUCT ){
    cout << "In PedStruct::PedStruct("<<filename<<","<<n_lines_to_skip<<endl;
  }
  ifstream IN (filename, ios::in);
  if( !IN ){
    cout << filename << " : no such file " << endl;
    exit(-1);
  }

  for( i=0; i< n_lines_to_skip; i++){
    IN.getline(ligne, L_LENGTH);
  }

  // Auto check if there are header lines; counting the number of 
  // lines.
  while(1){
    if( IN.eof() ) break;
    IN.getline(ligne, L_LENGTH );
    istringstream tampon (ligne);
    if( tampon >>foochar>>foolong>>foolong>>foolong>>fooint>>fooint){
      n_lignes++;
      header_line = 0;
    }
    else if( header_line ){
      n_header_lines++;
      if( DEBUG_PEDSTRUCT ){ 
	cout << "Header Lines: " << ligne << endl;
      }
    }
  }
    
  nind=n_lignes;
  ped = new char * [nind];
  id = new long [nind];
  fa = new long [nind];
  mo = new long [nind];
  sex= new int  [nind];
  extratext = new char * [nind];
  placeholder = new char * [nind];


  for( i=0; i<nind; i++){
    ped[i] = new char [ W_LENGTH ];
    extratext[i] = new char [ L_LENGTH ];
    extratext[i][0] ='\0';
    placeholder[i] = new char [1]; // will be deleted and re-newed
    placeholder[i][0]='\0';
    id[i] = fa[i] = mo[i] = 0;
    sex[i] = 0;
  }
  IN.clear();
  IN.seekg(0);
  
  if( !IN ){
    cout << "Can't open file " << filename << endl;
    exit(-1);
  }

  // skipping lines
  for( i=0; i< n_lines_to_skip; i++){
    IN.getline(ligne, L_LENGTH);
  }

  headerlines = new char * [1];

  if( n_header_lines ){
      
    delete [] headerlines;
    headerlines = new char * [ n_header_lines ];
  
    string HeaderLine;
      
    for(i=0;i<n_header_lines;i++){
      IN.getline(ligne, L_LENGTH);
      HeaderLine = ligne;
      headerlines[i]= new char[ HeaderLine.size()+1];
      HeaderLine.copy( headerlines[i], HeaderLine.size() );
      headerlines[i][  HeaderLine.size() ] = '\0';
    }
      
    if(DEBUG_PEDSTRUCT){
      cout << "Read " << n_header_lines  << " header lines on top ";
      cout << "of the first " << n_lines_to_skip << " skipped:\n";
    }
      
  }


  i=0;
  while(1){
    if( IN.eof() || i==nind ) break;
    IN.getline(ligne, L_LENGTH );
    istringstream tampon (ligne);
    if( tampon >> ped[i] >> id[i] >> fa[i] >> mo[i] >> sex[i] ){
      ostringstream tamponout;
      while( tampon >> foochar ){
	tamponout << foochar << "\t";
      }
      tamponout << '\0';
      tamponoutadr = tamponout.str();
      tamponoutadr.copy( extratext[i], tamponoutadr.length() );
      //      strcpy( extratext[i] ,tamponoutadr) ;      
    }
    i++; 
  }
    

  PedigreePositions();


  IN.close();
  delete ligne;
  delete foochar; 

    

}


////////////////////// Copy Constructor ///////////////////////////


PedStruct::PedStruct(PedStruct & P){
  
  if( DEBUG_PEDSTRUCT ){
    cout << "Entering :: PedStruct(PedStruct &)\n";
  }
      
  int i;
  
  nind=P.nind;
  n_header_lines = P.n_header_lines;
  ped = new char * [nind];
  id = new long [nind];
  fa = new long [nind];
  mo = new long [nind];
  sex = new int [nind];
  extratext = new char * [nind];
  placeholder = new char * [nind];
    
  for( i =0 ; i< nind; i++){
    ped[i] = new char [W_LENGTH];
    strcpy( ped[i], P.ped[i] );
    id[i]=P.id[i];
    fa[i]=P.fa[i];
    mo[i]=P.mo[i];
    sex[i]=P.sex[i];
    extratext[i]=new char [L_LENGTH];
    strcpy( extratext[i], P.extratext[i] );
    placeholder[i]=new char [L_LENGTH];
    strcpy( placeholder[i], P.extratext[i] );
  }
  headerlines = new char * [n_header_lines ];
  for( i=0; i< n_header_lines; i++){
    headerlines[i] = new char [L_LENGTH];
    strcpy( headerlines[i], P.headerlines[i] );
  }
  if( DEBUG_PEDSTRUCT ){
    cout << "Leaving... :: PedStruct(PedStruct &)\n";
  }
    

}

