/***********************************************************************

nucular -- a program that splits extended pedigrees into nuclear families
Copyright (C) 2005 Mathieu Lemire.

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**********************************************************************/


//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//******************************************************************************
//
//  Basic Class for reading a pedigree structure.
//  Ped Id Fa Mo Sex + the rest of line is stored
//  Header lines, if any, are stored


#include <iostream>
#include <fstream>
#include <string>
#include <sstream>

// The length of a line to read
#if !defined (L_LENGTH)
#define L_LENGTH 4096
#else
#if L_LENGTH < 4096
#undef L_LENGTH
#define  L_LENGTH 4096
#endif
#endif

// The length of a word to read
#if !defined (W_LENGTH)
#define W_LENGTH 64
#else
#if W_LENGTH < 64
#undef W_LENGTH
#define  W_LENGTH 64
#endif
#endif

#if !defined (DEBUG_PEDSTRUCT)
#define DEBUG_PEDSTRUCT 0
#endif

using namespace std;

class PedStruct 
{
  
  char ** ped;
  long * id;
  long * fa;
  long * mo;
  int * sex;
  char ** extratext;
  char ** placeholder; // if we want to write down extraextra text
  // through another application

  int nind;
  int npedigrees;
  int * pedpos;
  
  char ** headerlines;
  int n_header_lines;
  
public:

  PedStruct(){
    ;
  }
  PedStruct( char *, int = 0 ); // filename, n_lines_to_skip
  PedStruct( PedStruct &);

  ~PedStruct(){
    if( DEBUG_PEDSTRUCT ){
      cout <<"Entering ::~PedStruct()\n";
      cout <<  n_header_lines << " header lines" << endl;
    }
    delete [] ped;
    delete id;
    delete fa;
    delete mo;
    delete sex;
    delete pedpos;
    delete [] extratext;
    delete [] headerlines;
    delete [] placeholder;
    if( DEBUG_PEDSTRUCT ){
      cout <<"Leaving... ::~PedStruct()\n";
    }
  }


  char * Ped( int i ){
    CheckBound(i);
    return ped[i];
  }
  long Id( int i ){
    CheckBound(i);
    return id[i];
  }
  long Fa( int i ){
    CheckBound(i);
    return fa[i];
  }
  long Mo( int i ){
    CheckBound(i);
    return mo[i];
  }
  int Sex( int i ){
    CheckBound(i);
    return sex[i];
  }
  char * ExtraText( int i ){
    CheckBound(i);
    return extratext[i];
  }
  int NInd(){
    return nind;
  }
  char * HeaderLine( int i ){
    if( i>=0 && i< n_header_lines ){
      return headerlines[i];
    } else {
      cout << "HeaderLine " << i << " : out of bound" << endl;
      exit(-1);
    }
  }
  int NHeaderLines(){
    return n_header_lines;
  }
  void FillPlaceHolder( int i, char * foo, int size = L_LENGTH ){
    CheckBound(i);
    string Str (foo);
    delete placeholder[i];
    placeholder[i] = new char [ size+1 ];
    Str.copy( placeholder[i], size );
    placeholder[i][size]='\0';
  }
  void Write( char * filename, bool extrat = 0, bool placeh = 0 ){
    char z;
    int i;
    ifstream TEST (filename, ios::in);
    if (TEST){
      TEST.close();
      cout << filename << " already exists. Overwrite ? [y/n] ";
      cin >> z;
      if( z != 'y' && z != 'Y' ){
	exit(-1);
      }
    }
    ofstream OUT (filename, ios::out);
    for( i=0; i<nind;i++){
      OUT <<ped[i]<<"\t"<<id[i]<<"\t"<<fa[i]<<"\t"<<mo[i];
      OUT << "\t" << sex[i];
      if( extrat )
	OUT << "\t" << extratext[i];
      if( placeh )
	OUT << "\t" << placeholder[i];
      OUT << endl;
    }
  }
  void Write( ofstream & OUT ){
    int i;
    for( i=0; i<nind;i++){
	OUT <<ped[i]<<"\t"<<id[i]<<"\t"<<fa[i]<<"\t"<<mo[i]<<"\t";
	OUT << sex[i] << "\t";
	OUT << extratext[i] << endl;
    }
  }
 
  bool CheckBound( int i ){
    if( i<0 || i>=nind ){
      cout << "Individual " << i << " : out of bounds\n";
      exit(-1);
    }
    return 1;
  }

  int FindId( char * pedname, long idname ){ 
    // return index i, or -1 if not found
    int i;
    string p (pedname);
    for(i=0; i<nind; i++){
      if( id[i] == idname ){
	string pi (ped[i]);
	if( p==pi ){
	  return i;
	}
      }
    }
    return -1;
  }
	


  void PedigreePositions(){  // Called in Constructor

    if( DEBUG_PEDSTRUCT ){
      cout << "In PedStruct:: PedigreePositions()"<<endl;
    }
    int i,j,k;
    string lastped;
    string currped;
    lastped = '\0';    
    npedigrees=0;
    for( i=0; i<nind; i++){
      currped = ped[i];
      if( currped != lastped ){
	npedigrees++;
	lastped = currped;
      }
    }

    pedpos = new int [npedigrees+1];
    lastped = '\0';  
    j=0;
    for( i=0; i<nind; i++){
      currped = ped[i];
      if( currped != lastped ){
	pedpos[j] = i;
	lastped=currped;
	j++;
      }
    }
    pedpos[ npedigrees ] = nind;
    if( j!=npedigrees ){
      cout << "npedigrees error \n";
      exit(-1);
    }

    // Checking that the pedigree is ordered;

    /*
    for( i=0; i<npedigrees-1;i++){
      for( j=i+1; j<npedigrees; j++){
	lastped=ped[pedpos[i]];
	currped=ped[pedpos[j]];
	if( lastped == currped ){
	  cout << "Pedigree file has not been sorted :" << endl;
	  for( k=i; k<=j; k++){
	    cout << "line :" << pedpos[k] << "    pedigree name :";
	    cout << ped[pedpos[k]] << endl;
	  }
	}
      }
    }
    */
     if( DEBUG_PEDSTRUCT ){
       cout << "Counted " << npedigrees << " pedigrees\n";
       cout << "Leaving PedStruct:: PedigreePositions()"<<endl;
     }

 
  }
   
  int NPedigrees(){
    return npedigrees;
  }

  int Pedpos( int i ){ // i is a pedigree number
    if( i<0 || i>npedigrees ){
      cout << "Pedpos() : Index out of bound" << endl;
      exit(-1);
    }
    return pedpos[i];
  }

  void WritePed( int i, ofstream & OUT ){
    int j;
    if( i<0 || i>npedigrees ){
      cout << "WritePed() : Index out of bound" << endl;
      exit(-1);
    }
    for( j = Pedpos(i); j<Pedpos(i+1); j++){
      OUT << Ped(j) << "\t" << Id(j);
      OUT << "\t" << Fa(j) << "\t" << Mo(j);
      OUT << "\t" << Sex(j);
      OUT << "\t" << ExtraText(j);
      OUT << endl;
    }
  }
  void WritePed( int i ){
    int j;
    if( i<0 || i>npedigrees ){
      cout << "WritePed() : Index out of bound" << endl;
      exit(-1);
    }
    for( j = Pedpos(i); j<Pedpos(i+1); j++){
      cout << Ped(j) << "\t" << Id(j);
      cout << "\t" << Fa(j) << "\t" << Mo(j);
      cout << "\t" << Sex(j);
      cout << "\t" << ExtraText(j);
      cout << endl;
    }
  }

      


};

