/***********************************************************************

nucular -- a program that splits extended pedigrees into nuclear families
Copyright (C) 2005 Mathieu Lemire.

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**********************************************************************/



#include "PedStructByFam.hh"


PedStructByFam::PedStructByFam( char * filename, 
				int n_lines_to_skip,
				bool odd_id ):PedStruct( filename, n_lines_to_skip){
  
  int i;
  
  if( DEBUG_CHECKPEDSTRUCT ){
    cout << "In PedStructByFam::PedStructByFam(";
    cout << filename << "," << n_lines_to_skip << ")"<<endl;
  }

  is_father = new bool [ NInd() ];
  is_mother = new bool [ NInd() ];
  unlinked = new bool [ NInd() ];

  pednames = new char * [ NInd() ];
  
  for(i=0;i< NInd();i++){
    pednames[i] = new char [W_LENGTH];
    is_father[i]=0;
    is_mother[i]=0;
    unlinked[i]=0;
     }  
  n_families = 0;

  female_has_odd_id = odd_id;
 
  FindParents();
  
}


void PedStructByFam::FindParents(){
  
  int i,j,k,id1,id2;

  for( i=0; i<NPedigrees(); i++){
    id1=Pedpos(i);
    id2=Pedpos(i+1);
    for( j=id1; j<id2; j++){      
      if( Fa(j) != 0 ){ // looking for the father
	for( k=id1; k<id2; k++){
	  if( Id(k) == Fa(j) ){
	    is_father[k] = 1;
	    if(DEBUG_CHECKPEDSTRUCT)
	      cout << Ped(k) << "-" << Id(k) << " is a father\n";

	    break;
	  }
	}
      }
      if( Mo(j) != 0 ){ // looking for the mother
	for( k=id1; k<id2; k++){
	  if( Id(k) == Mo(j) ){
	    is_mother[k] = 1;
	    break;
	  }
	}
      } 
    }
  }
  
}


bool PedStructByFam::CheckUnlinked(){
  int i;
  bool error = 0;
  for( i=0; i<NInd();i++){  
    if( ( Fa(i)*Mo(i)==0)  && (is_mother[i]+is_father[i]==0) ){
      cout << Ped(i) << "-" << Id(i) << " " << Fa(i) << " "<<Mo(i) ;
      cout << "is unlinked" << endl;
      unlinked[i]=1;
      error=1;
    }
  }
  return error;
}
   

bool PedStructByFam::CheckSex(){
  
  int i;
  bool error=0;

  for( i=0; i<NInd();i++){  
    if( Sex(i) != 1 && Sex(i) != 2 && !unlinked[i] ){
      cout << Ped(i) << "-" << Id(i) << " ";
      cout << "have undefined sex (" << Sex(i) << ")\n";
      error = 1;
    }
    if( (Fa(i)+Mo(i)>0) && Fa(i)*Mo(i)==0 ){ // both parents defined
      cout << Ped(i) << "-" << Id(i) << " ";
      cout << "has one missing parent \n";
      error=1;
    }
    if( is_father[i] && Sex(i) == 2 ){ // check for correct sex
      cout << Ped(i) << "-" << Id(i) << " ";
      cout << "is father but female\n";
      error=1;
    } 
    if( is_mother[i] && Sex(i) == 1 ){ // correct sex
      cout << Ped(i) << "-" << Id(i) << " ";
      cout << "is mother but male\n";
      error=1;
    }
    if(  female_has_odd_id && (Id(i)+Sex(i)) % 2 == 0 ){ 
      cout << Ped(i) << "-" << Id(i) << " "; // according to the standard that males have even number
      cout << "might have the wrong sex defined (" << Sex(i) << ")\n";
      error=1;
    }
  }
  
  return error;

}






bool PedStructByFam::CheckStructure(){
  
 
  
  int i;
  bool error=0;

 
  if( DEBUG_CHECKPEDSTRUCT ){ 
    cout << "Number of families : " << n_families << endl;    
    for( i=0; i<n_families; i++){
      cout << "Stored " << pednames[i] << " as family number " << i ;
      cout << " -- Pedpos() " << Pedpos(i) << " to " <<Pedpos(i+1) << endl;
    }
  }
  
 
  error = CheckUnlinked();
  error +=CheckSex();

  return error;

}


