/***********************************************************************

nucular -- a program that splits extended pedigrees into nuclear families
Copyright (C) 2005 Mathieu Lemire.

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**********************************************************************/



#include "PedStruct.hh"

#if !defined (DEBUG_CHECKPEDSTRUCT)
#define DEBUG_CHECKPEDSTRUCT 0
#endif

class PedStructByFam : public PedStruct
{

  int n_families;
  char ** pednames;

  bool * is_father;
  bool * is_mother;
  bool * unlinked;
  
  bool female_has_odd_id;

 public:
  
  PedStructByFam(){
    ;
  }
  PedStructByFam( char *, int =0, bool=0);

  PedStructByFam( PedStructByFam & P ) : PedStruct (P){
    if( DEBUG_CHECKPEDSTRUCT ){
      cout <<"Entering ::PedStructByFam( PedStructByFam & )\n";
    }
    int i;
    n_families = P.n_families;
    female_has_odd_id = P.female_has_odd_id;
    pednames = new char * [P.NInd()];
    is_father = new bool [P.NInd()];
    is_mother = new bool [P.NInd()];
    unlinked = new bool [P.NInd()];
    for( i=0; i<P.NInd(); i++){
      pednames[i] = new char [W_LENGTH+10000];
      strcpy(pednames[i],P.pednames[i]);
      is_father[i] = P.is_father[i];
      is_mother[i] = P.is_mother[i];
      unlinked[i] = P.unlinked[i];
         }
    if( DEBUG_CHECKPEDSTRUCT ){
      cout <<"Leaving... ::PedStructByFam( PedStructByFam & )\n";
    }
  }
  ~PedStructByFam(){
    if( DEBUG_CHECKPEDSTRUCT ){
      cout <<"Entering ::~PedStructByFam()\n";
    }
    delete is_father;
    delete is_mother;
    delete unlinked;
    delete [] pednames;
       if( DEBUG_CHECKPEDSTRUCT ){
      cout <<"Leaving... ::~PedStructByFam()\n";
    }

  }
  
  void FindParents();
  bool CheckUnlinked();
  bool CheckSex();
  bool CheckStructure();

  bool IsFather( int i ){
    if( i>=0 && i<=NInd() ){
      return is_father[i];
    } else {
      cout << "PedStructByFam::IsFather( " << i << ") : out of bounds\n";
      exit(-1);
    }
  }
  bool IsMother( int i ){
    if( i>=0 && i<=NInd() ){
      return is_mother[i];
    } else {
      cout << "PedStructByFam::IsMother( " << i << ") : out of bounds\n";
      exit(-1);
    }
  }
  
     
   


};    
      
  
    


