/***********************************************************************

nucular -- a program that splits extended pedigrees into nuclear families
Copyright (C) 2005 Mathieu Lemire.

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**********************************************************************/



#if !defined (INCLUDE_PEDSTRUCTBYNUCLEARFAM)
#include "PedStructByNuclearFam.hh"
#define INCLUDE_PEDSTRUCTBYNUCLEARFAM
#endif


PedStructByNuclearFam::PedStructByNuclearFam(char * filename, 
					     int  n_lines_to_skip, 
					     bool odd_id):PedStructByFam( filename, n_lines_to_skip, odd_id){
  
  int i, j,k, f=0,m=0 ;
  int * usedaskid = new int [NInd()]; // 1 for founders
  int * usedasparent = new int [NInd()];
 
  int allusedaskid, proband;
  int currnuc=-1;


  for( i=0; i< NInd(); i++){
    usedaskid[i] = 0;
    usedasparent[i]=0;

    //cout << i << " " << Fa(i) << " " <<  IsFather(i) << " " <<  IsMother(i)<<endl;

    if( Fa(i) == 0 ) usedaskid[i]=1;
    if( IsFather(i) ) f++; // number of dad found
    if( IsMother(i) ) m++; // nmumber of mom found
  }
  if( f==m ){
    nnuclearfamilies = f;
  } else {
    cout << "Different number of mothers and fathers in file " << filename;
    cout << endl;

    if( f>m ) f=m;  //smaller number defines number of nuc fam

    nnuclearfamilies = f;

  }

  
 
  nuclearfamilysizes = new int [f];
  nuclearfamilylabels = new int * [f];
  isparent = new int * [f];
  for( i=0; i<f;i++){
    nuclearfamilylabels[i]=new int [NInd()]; // bigger than needed;
    isparent[i]=new int [NInd()]; 
    for( j=0; j<NInd(); j++){
      nuclearfamilylabels[i][j]=-1;
      isparent[i][j]=0;
    }
  } 
  
  for( i=0; i<NPedigrees(); i++){
    allusedaskid = 0; // as kid
    while(!allusedaskid){
      
      // looking for a "proband"
      for( j=Pedpos(i); j<Pedpos(i+1); j++){
	if( Fa(j) != 0  &&  !usedaskid[j] ){
	  usedaskid[j]=1;
	  proband=j;
	  currnuc++;
	  break;
	}
      }
      if( j==Pedpos(i+1)) break;
   
      // looking for parents of proband
      f=-1; m=-1;
      for( j=Pedpos(i); j<Pedpos(i+1); j++){
	if( Fa( proband )==Id(j) ){
	  nuclearfamilylabels[currnuc][0] = j;
	  usedasparent[j]=1;
	  isparent[currnuc][0]=1;
	  
	  f=j;
	}
	if( Mo( proband )==Id(j) ){
	  nuclearfamilylabels[currnuc][1] = j;
	  usedasparent[j]=1;
	  isparent[currnuc][1]=2;
	  m=j;
	}
	
      }
      
      nuclearfamilysizes[currnuc]=2;
      // looking for sibs
      for( j=Pedpos(i); j<Pedpos(i+1); j++){
	if( Fa(j)==Id(f) || Mo(j)==Id(m) ){ // OR to allow for half-sibs

	  nuclearfamilylabels[currnuc][  nuclearfamilysizes[currnuc] ]=j;
	  nuclearfamilysizes[currnuc]++;
	  usedaskid[j]=1;
	
	  if( Fa(j) != Id(f) || Mo(j)!=Id(m) ){
	    // looking for the father (mother) of j if half-sib
	    for( k=Pedpos(i); k<Pedpos(i+1); k++){
	      if( Fa(j)==Id(k) && !usedasparent[k] ){
		nuclearfamilylabels[currnuc][  nuclearfamilysizes[currnuc] ]=k;
		
		usedasparent[k]=1;
		isparent[currnuc][ nuclearfamilysizes[currnuc]]=1;nuclearfamilysizes[currnuc]++;
		break;
	      }
	      if( Mo(j)==Id(k) && !usedasparent[k] ){
		nuclearfamilylabels[currnuc][  nuclearfamilysizes[currnuc] ]=k;
		
		usedasparent[k]=1;
		isparent[currnuc][ nuclearfamilysizes[currnuc] ]=2;nuclearfamilysizes[currnuc]++;
		break;
	      }
	    }
	  }

	}
      } // end looking for sibs


      allusedaskid = 1;
      for( j=Pedpos(i); j<Pedpos(i+1); j++){
	if( !usedaskid[j] ){
	  allusedaskid=0;
	  break;
	}
	  
      }

    }    
        
  }

  delete usedaskid;
  delete usedasparent;

    
}
