/***********************************************************************

nucular -- a program that splits extended pedigrees into nuclear families
Copyright (C) 2005 Mathieu Lemire.

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at your
option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**********************************************************************/


#if !defined( INCLUDE_PEDSTRUCTBYNUCLEARFAM )
#include "PedStructByNuclearFam.hh"
#define INCLUDE_PEDSTRUCTBYNUCLEARFAM 
#endif

#include <iostream>
#include <fstream>
#include <string>
#include<sstream>


#define DEBUG 1
#define DCOUT if( DEBUG ) cout

main(int argc, char * argv[]){

 
  bool dummyfathers=0;
  bool dummymothers=0;
  bool allaffected=0;
  int i,j,k, nsibs, sib, idx;
  char * extra;
  char bidon [W_LENGTH];
  int bit=0;
  string filename_str, t_str;

  if( argc < 2){
    cout << "Usage:";
    cout << "nucular [-fma] <pedfile>"<<endl;
    cout << " -f : create dummy fathers" << endl;
    cout << " -m : create dummy mothers" << endl;
    cout << " -a : mark all as affected" << endl;
    cout << "nucular extracts nuclear families " << endl;
    exit(-1);
  }

  for(i=1; i<argc; i++){
    string arg (argv[i]);
    if( arg.find("-f")==0){
      dummyfathers=1;
      if( dummymothers ){
	cout << "Only one of -f or -m is allowed"<<endl;
	exit(-1);
      }
    }
    if( arg.find("-m")==0){
      dummymothers=1;
      if( dummyfathers ){
	cout << "Only one of -f or -m is allowed"<<endl;
	exit(-1);
      }
    }
    if(  arg.find("-a")==0){
      allaffected =1 ;
    }
  }

  ostringstream oooout;
  if( dummyfathers ) oooout << "dummyf_";
  if( dummymothers ) oooout << "dummym_";
  oooout << "nucular_" << argv[argc-1]  << '\0';

  filename_str = oooout.str();
  char * filename_out = new char [filename_str.length()];
  filename_str.copy(   filename_out  , filename_str.length() );
  ofstream OUT (filename_out, ios::out);

  DCOUT << "Reading file " << argv[argc-1] << endl;
  PedStructByNuclearFam PED (argv[argc-1]);
  DCOUT << "Done" << endl;

  DCOUT << "NNucFam " << PED.NNuclearFamilies()<< endl;

  for( i=0; i<PED.NNuclearFamilies(); i++){

    // counting the number of sibs
    nsibs=0;
    for( j=0; j<PED.NuclearFamilySizes(i); j++){
      if( !PED.IsParent(i,j) ) nsibs++;
      
      if( DEBUG ){
	idx= PED.NuclearFamilyLabels(i,j);
	//cout << PED.Ped(idx) << "-" << PED.Id(idx) << " " << PED.IsParent(i,j);
	//cout << endl;
      }

    }


    // we're supposed to have nsibs parents in duplicates
    // won't work well for families having half sibs already
    if( dummyfathers ){
      sib=0;
      for( j=0; j<PED.NuclearFamilySizes(i); j++){
	idx= PED.NuclearFamilyLabels(i,j);


	// writing extra text
	istringstream tampon (PED.ExtraText(idx) );
	ostringstream t;
	if( allaffected ){
	  tampon >> bidon;
	  t << "2";
	}
	else {
	  tampon >> bidon;
	  t << bidon;
	}
	while( tampon >> bidon ){
	  t << "\t" << bidon;
	}
	//t << '\0';
	t_str = t.str();
	if( !PED.IsParent(i,j) ){
	  sib++;
	  OUT << PED.Ped(idx) << "_" << i+1 << "\t";
	  OUT << PED.Id(idx) << "\t";
	  OUT << PED.Fa( idx ) << "0" << sib  << "\t";
	  OUT << PED.Mo(idx) << "\t";
	  OUT << PED.Sex(idx) << "\t";

	  OUT << t_str << endl;	

	  if( 0 ){
	  bit= 2*nsibs-nsibs-1;
	  for( k=1; k<=nsibs && bit<20-2*nsibs ; k++){	
	    if( k!=sib ){
	      OUT << PED.Ped(idx) << "_" << i+1 << "\t";
	      OUT << PED.Id(idx) << "0" << k << "\t";
	      OUT << PED.Fa( idx ) << "0" << k  << "\t";
	      OUT << PED.Mo(idx) << "\t";
	      OUT << PED.Sex(idx) << "\tZERO";
	      OUT << t_str << endl;	
	    
	      bit+=nsibs*2;
	    }
	  }
	  }

	} else if( PED.IsParent(i,j) == 1 ){ // writin

	  for( k=0; k<nsibs; k++){	
	    
	    OUT <<  PED.Ped(idx) << "_" << i+1 << "\t";
	    OUT << PED.Id(idx) << "0" << k+1 << "\t";
	    OUT << "0\t0\t";
	    OUT << PED.Sex(idx) << "\t";

	    OUT << t_str << endl;	
	  }
	} else if( PED.IsParent(i,j) == 2 ){ //  writing single mother
	   
	    OUT <<  PED.Ped(idx) << "_" << i+1 << "\t";
	    OUT << PED.Id(idx) << "\t";
	    OUT << "0\t0\t";
	    OUT << PED.Sex(idx) << "\t";
	    
	    OUT << t_str << endl;

	  
	}
      }
    } else if( dummymothers ){
      sib=0;
      for( j=0; j<PED.NuclearFamilySizes(i); j++){
	idx= PED.NuclearFamilyLabels(i,j);
	  // writing extra text
	istringstream tampon (PED.ExtraText(idx));
	ostringstream t;
	if( allaffected ){
	  tampon >> bidon;
	  t << "2";
	}
	else {
	  tampon >> bidon;
	  t << bidon;
	}
	while( tampon >> bidon ){
	  t << "\t" << bidon;
	}
	//	t << '\0';
	t_str = t.str();
	if( !PED.IsParent(i,j) ){
	  sib++;
	 
	  OUT << PED.Ped(idx) << "_" << i+1 << "\t";
	  OUT << PED.Id(idx) << "\t";
	  OUT << PED.Fa(idx) << "\t";
	  OUT << PED.Mo( idx ) << "0" << sib  << "\t";
	  OUT << PED.Sex(idx) << "\t";
	  OUT << t_str << endl;	

	  if(0){
	  bit= 2*nsibs-nsibs-1;
	  for( k=1; k<=nsibs && bit<20-2*nsibs ; k++){	
	    if( k!=sib ){
	       OUT << PED.Ped(idx) << "_" << i+1 << "\t";
	       OUT << PED.Id(idx) << "0"<< k  << "\t";
	       OUT << PED.Fa(idx) << "\t";
	       OUT << PED.Mo( idx ) << "0" << k  << "\t";
	       OUT << PED.Sex(idx) << "\tZERO";
	       OUT << t_str << endl;	
	       bit+=nsibs*2;
	       
	    }
	  }
	  }



	}
	if( PED.IsParent(i,j) == 2 ){ // writing multiple mothers
	  
	  for( k=0; k<nsibs; k++){	    
	    OUT <<  PED.Ped(idx) << "_" << i+1 << "\t";
	    OUT << PED.Id(idx) << "0" << k+1 << "\t";
	    OUT << "0\t0\t";
	    OUT << PED.Sex(idx) << "\t";
	    OUT << t_str << endl;	
	  }
	}
	
	if( PED.IsParent(i,j) == 1 ){ //  writing single father
	  
	  OUT <<  PED.Ped(idx) << "_" << i+1 << "\t";
	  OUT << PED.Id(idx) << "\t";
	  OUT << "0\t0\t";
	  OUT << PED.Sex(idx) << "\t";
	  OUT << t_str << endl;

	  
	}
      }
    } else {
      // just write down the nuclear families

      for( j=0; j<PED.NuclearFamilySizes(i); j++){
	idx= PED.NuclearFamilyLabels(i,j);
	istringstream tampon (PED.ExtraText(idx) );
	ostringstream t;
	if( allaffected ){
	  tampon >> bidon;
	  t << "2";
	}
	else {
	  tampon >> bidon;
	  t << bidon;
	}
	while( tampon >> bidon ){
	  t << "\t" <<  bidon;
	}
	//t << '\0';

	t_str = t.str();
	
	if( !PED.IsParent(i,j) ){
   
	  OUT << PED.Ped(idx) << "_" << i+1 << "\t";
	  OUT << PED.Id(idx) << "\t";
	  OUT << PED.Fa( idx ) << "\t";
	  OUT << PED.Mo(idx) << "\t";
	  OUT << PED.Sex(idx) << "\t";
	  OUT << t_str << endl;	
	  
	} else if( PED.IsParent(i,j) ){ 
	  OUT <<  PED.Ped(idx) << "_" << i+1 << "\t";
	    OUT << PED.Id(idx) << "\t";
	    OUT << "0\t0\t";
	    OUT << PED.Sex(idx) << "\t";
	    OUT <<t_str  << endl;	
	}
	
      }

    }






  }

  OUT.close();

  
}
  
